#define NK_INCLUDE_FIXED_TYPES
#define NK_INCLUDE_STANDARD_IO
#define NK_INCLUDE_STANDARD_VARARGS
#define NK_INCLUDE_DEFAULT_ALLOCATOR
#define NK_IMPLEMENTATION
#define NK_XLIB_IMPLEMENTATION
#define STB_IMAGE_IMPLEMENTATION
#define NK_XLIB_INCLUDE_STB_IMAGE

#include            <nuklear.h> // GUI lib
#include              <ctype.h> // toupper()
#include "lib/getUsedImgPath.h"
#include   "lib/nuklear_xlib.h"
#include     "lib/startsWith.h"
#include      "lib/dirExists.h"
#include          "lib/langs.h"
#include          "lib/style.c"
#include           "lib/ccvt.h"

#define DTIME          20

#define WINDOW_WIDTH  500
#define WINDOW_HEIGHT 350
#define ROW_HEIGHT     30

#define BUFFERSIZE    256

#define PROGRAM_NAME "cURRENCY cONvERtER"

typedef struct XWindow XWindow;
struct XWindow {
           Display              * dpy             ;
           Window                 root            ;
           Visual               * vis             ;
           Colormap               cmap            ;
           XWindowAttributes      attr            ;
           XSetWindowAttributes   swa             ;
           Window                 win             ;
           int                    screen          ;
           XFont                * font            ;
  unsigned int                    width           ;
  unsigned int                    height          ;
           Atom                   wm_delete_window;
};

static void     die      (const char *fmt, ...) {
    va_list  ap              ;
    va_start(ap, fmt        );
    vfprintf(stderr, fmt, ap);
    va_end  (ap             );
    fputs   ("\n", stderr   );
    exit    (EXIT_FAILURE   );
}

static long     timestamp() {
    struct timeval tv                                               ;
    if     (gettimeofday(&tv, NULL) < 0) return 0                   ;
    return (long) ((long) tv.tv_sec * 1000 + (long) tv.tv_usec/1000);
}

static void     sleep_for(long t) {
    struct timespec req                           ;
    const  time_t   sec         = (int)(t/1000)   ;
    const  long     ms          = t - (sec * 1000);
                    req.tv_sec  = sec             ;
                    req.tv_nsec = ms * 1000000L   ;
    while (-1 == nanosleep(&req, &req))           ;
}

static nk_flags textin   (struct nk_context * context, char * buffer) {
  return nk_edit_string_zero_terminated(context, NK_EDIT_BOX,
                                        buffer, BUFFERSIZE,
                                        nk_filter_ascii);
}

static char    *toupperS (char * string) {
  // convert string to uppercase
  for (unsigned long int i = 0; i < strlen(string); ++i)
    string[i] = toupper(string[i]);

  return string;
}

static int      lookup   (struct nk_context * context, nk_flags event, struct currency * currencies, int number, char * text, char * search) {
  nk_label(context, text, NK_TEXT_LEFT);
  event = textin(context, search);

  for  (int i = 0; i < number; ++i)
    if (startsWith(currencies[i].name, toupperS(search)) == 1)
      return i;

  return     0;
}

enum {COMBOBOX, SEARCH}             ;

static char   * flagPath (char * imgPath, struct currency * currencies, int val, char * path) {
  // clear "path"
  strcpy(path, "\0"                 );

  // construct flag path
  strcat(path,  imgPath             );
  strcat(path, "flags/"             );
  strcat(path,  currencies[val].name);
  strcat(path, ".bmp"               );

  return path;
}

static void     searchin (struct nk_context * context, nk_flags event, struct currency * currencies, int currNum, struct nk_vec2 size, int * val, char ** options, char * buffer, int op, char * fieldLabel, char * searchLabel, char * imgPath, struct nk_image flag, char * path) {

  nk_layout_row_dynamic  (context,     ROW_HEIGHT,              1                       );

  nk_label               (context,     fieldLabel,              NK_TEXT_LEFT            );
  if (op == COMBOBOX) {
    nk_layout_row_dynamic(context,     ROW_HEIGHT,              2                       );

    nk_combobox          (context,    (const char **) options,  currNum, &*val, 20, size);
  }
  else {
    *val  = lookup                       (context,     event, currencies, currNum,
                                          searchLabel, buffer                           );

    nk_layout_row_dynamic(context,     ROW_HEIGHT,              2                       );

    nk_text              (context,     currencies[*val].name,   3,
                          NK_TEXT_CENTERED                                              );

  }

  // set flag
     flag = nk_xsurf_load_image_from_file(flagPath(imgPath, currencies, *val, path)     );

  // show flag
  nk_image               (context,     flag                                             );
}

int main() {
         long         dt                 ;
         long         started            ;
         int          running        = -1;
         XWindow      xw                 ;
  struct nk_context * ctx                ;
         char         buffer [BUFFERSIZE] = "\0", // initialise as empty string
                      search [BUFFERSIZE] = "\0",
                      search1[BUFFERSIZE] = "\0";

  // X11 setup
  memset              (&xw,    0,      sizeof(xw)             )              ;

  xw.dpy              = XOpenDisplay     (NULL                              );
  if (!xw.dpy)
    die("Could not open display; perhaps $DISPLAY is not set?")              ;

  xw.root             = DefaultRootWindow(xw.dpy                            );
  xw.screen           = XDefaultScreen   (xw.dpy                            );
  xw.vis              = XDefaultVisual   (xw.dpy, xw.screen                 );
  xw.cmap             = XCreateColormap  (xw.dpy, xw.root, xw.vis, AllocNone);

  xw.swa.colormap     = xw.cmap                                              ;
  xw.swa.event_mask   =
        ExposureMask      | KeyPressMask      | KeyReleaseMask    |
        ButtonPress       | ButtonReleaseMask | ButtonMotionMask  |
        Button1MotionMask | Button3MotionMask | Button4MotionMask |
        Button5MotionMask | PointerMotionMask | KeymapStateMask              ;

  xw.win              = XCreateWindow    (xw.dpy,  xw.root,   0, 0,
                                          WINDOW_WIDTH *2.7,
                                          WINDOW_HEIGHT*1.5,  0,
                         XDefaultDepth(xw.dpy, xw.screen), InputOutput,
                         xw.vis, CWEventMask | CWColormap, &xw.swa)          ;

  XStoreName          (xw.dpy, xw.win, PROGRAM_NAME           )              ;
  XMapWindow          (xw.dpy, xw.win                         )              ;
  xw.wm_delete_window = XInternAtom      (xw.dpy, "WM_DELETE_WINDOW", False) ;
  XSetWMProtocols     (xw.dpy, xw.win, &xw.wm_delete_window, 1)              ;
  XGetWindowAttributes(xw.dpy, xw.win, &xw.attr               )              ;
  xw.width            = (unsigned int) xw.attr.width                         ;
  xw.height           = (unsigned int) xw.attr.height                        ;

  // GUI setup
  xw.font = nk_xfont_create(xw.dpy,  "fixed"                                        );
  ctx     = nk_xlib_init   (xw.font,  xw.dpy, xw.screen, xw.win, xw.width, xw.height);

  // style
  set_style(ctx, THEME_DARK);

  // get currencies
  struct currency * currencies = getValues (           );
         int        currNum    = countLines(getERates());
         char     * options[currNum]                    ;

  // combobox options
  for (int i = 0; i < currNum; ++i)
    options[i] = currencies[i].name;

  // currency pick combobox stuff
  static int           init     =  0                               ,
                       targ     =  0                               , // selected currencies
                       temp     =  0                               ;
  struct nk_vec2       size     = {140                             , // dropdown menu width
                                   400}                            , // =             height
                   langSize     = {100                             ,
                                   100}                            ;

  // other vars
         nk_flags      event    =  0                               ;
         char          out     [BUFFERSIZE                   ]     ;
         long double   result                                      ;
  static int           op       = COMBOBOX                         ,
                       lang     = ENGLISH                          ; // default lang

  // images
  struct nk_image      map                                         ,
                       flag     = nk_xsurf_load_image_from_file("");
         char        * imgPa    = getUsedImgPath()                 ;
         char        * fName    = getMapImgName ()                 ;
         char          mapP    [strlen(imgPa) + strlen(fName)]     , // make enough room
                       flagPath[BUFFERSIZE]                        ;

  strcpy(mapP, imgPa);
  strcat(mapP, fName);
  map = nk_xsurf_load_image_from_file(mapP);

  // initialise labels in different languages
  initLabels();

  while (running) {
    // input
    XEvent evt                  ;
           started = timestamp();

    nk_input_begin        (ctx                             );
    while (XPending(xw.dpy)) {
      XNextEvent          (xw.dpy, &evt                    );
      if (evt.type == ClientMessage)  goto     cleanup      ;
      if (XFilterEvent(&evt, xw.win)) continue              ;
      nk_xlib_handle_event(xw.dpy,  xw.screen, xw.win, &evt);
    }
    nk_input_end          (ctx                             );

    // GUI
    if (nk_begin(ctx, PROGRAM_NAME,
                 nk_rect(-1, -1, WINDOW_WIDTH, WINDOW_HEIGHT         ),
                         NK_WINDOW_MOVABLE | NK_WINDOW_TITLE         |
                         NK_WINDOW_BORDER  | NK_WINDOW_NO_SCROLLBAR  )) {

      nk_layout_row_dynamic  (ctx,  WINDOW_HEIGHT - 45,      3                       );

      if (nk_group_begin(ctx, "group1", NK_WINDOW_BORDER)) { // group 1
        searchin             (ctx,  event,                   currencies,
                              currNum,                       size,
                             &init,                          options,  search,
                              op,
                              labels[lang].initial,          labels[lang].search,
                              imgPa,
                              flag,                          flagPath                );

        nk_layout_row_dynamic(ctx,  ROW_HEIGHT,              1                       );

        nk_label             (ctx,  labels[lang].value,      NK_TEXT_LEFT            );
        event  = textin      (ctx,  buffer                                           );

        nk_group_end         (ctx                                                    );
      }

      if (nk_group_begin(ctx, "group2", 0               )) { // group 2
        nk_layout_row_dynamic(ctx,  ROW_HEIGHT,              2                       );

        if (nk_option_label(ctx, labels[lang].select, op == COMBOBOX)) op = COMBOBOX;
        if (nk_option_label(ctx, labels[lang].search, op == SEARCH  )) op = SEARCH  ;

        nk_layout_row_dynamic(ctx,  ROW_HEIGHT + 10,         1                       );

        // invert button
        if (nk_button_label(ctx, labels[lang].invert)) {
          if (op == COMBOBOX) {
            temp = init;
            init = targ;
            targ = temp;
          }
          else {
            char  tempS[BUFFERSIZE] = "\0";
            strcpy(tempS,   search )      ;
            strcpy(search,  search1)      ;
            strcpy(search1, tempS  )      ;
          }
        }

        nk_layout_row_dynamic(ctx,  ROW_HEIGHT + 10,         1                       );

        // update eRates button
        if (nk_button_label(ctx,  labels[lang].update       )) {
          system             ("update-eRates.sh"                                     );
          currencies = getValues(                                                    );
        }

        nk_layout_row_dynamic(ctx,  ROW_HEIGHT + 65,         1                       );

        nk_layout_row_static (ctx,  30,        100,          1                       );

        nk_label             (ctx,  labels[lang].language,    NK_TEXT_LEFT            );
        nk_combobox          (ctx,  languages, sizeof(languages)/sizeof(languages[0]) ,
                             &lang, 20,        langSize                              );


        nk_group_end         (ctx                                                    );
      }

      if (nk_group_begin(ctx, "group3", NK_WINDOW_BORDER)) { // group 3
        searchin             (ctx,  event,                   currencies,
                              currNum,                       size,
                             &targ,                          options,  search1,
                              op,
                              labels[lang].target,           labels[lang].search,
                              imgPa,
                              flag,                          flagPath                );

        // convert
        result = convert(atof(buffer), currencies[init].name, currencies[targ].name,
                         0,            currencies,            currNum,
                         1 /*, because we're in a GUI */                             );

        // convert result to string
        sprintf              (out, "%Lf",     result                                 );

        nk_layout_row_dynamic(ctx,  ROW_HEIGHT,              1                       );

        nk_label             (ctx,  labels[lang].value,      NK_TEXT_RIGHT           );
        if (nk_widget_is_hovered(ctx))
          nk_tooltip         (ctx,  out                                              );
        nk_text              (ctx,  out,                     strlen(out),
                              NK_TEXT_CENTERED                                       );

        nk_group_end         (ctx                                                    );
      }
      nk_end                 (ctx                                                    );
    }

    if (nk_begin(ctx, labels[lang].map,
                 nk_rect(WINDOW_WIDTH + 10,  -1, // so that the new window spawns
                                                 // right next to the previous
                                                 // one
                         MAP_WIDTH,          MAP_HEIGHT + 42 /* title bar */),
                         NK_WINDOW_MOVABLE | NK_WINDOW_TITLE                |
                         NK_WINDOW_BORDER  | NK_WINDOW_NO_SCROLLBAR         )) {
      nk_layout_row_dynamic(ctx, MAP_HEIGHT, 1);

      nk_image             (ctx, map          );

      nk_end               (ctx               );
    }

    if (nk_window_is_hidden(ctx, PROGRAM_NAME))
      break;

    // drawing
    XClearWindow  (xw.dpy, xw.win            );
    nk_xlib_render(xw.win, nk_rgb(28, 33, 35));
    XFlush        (xw.dpy                    );

    // timing
    dt = timestamp() - started;
    if (dt < DTIME)
      sleep_for(DTIME - dt)   ;
  }

  cleanup: {
    nk_xfont_del    (xw.dpy, xw.font);
    nk_xlib_shutdown(               );
    XUnmapWindow    (xw.dpy, xw.win );
    XFreeColormap   (xw.dpy, xw.cmap);
    XDestroyWindow  (xw.dpy, xw.win );
    XCloseDisplay   (xw.dpy         );
  }
  return             0               ;
}
